#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*
    getarg --- Parse Argument line

    Parameters: argc	int	Argument count, elements of argv[], 
				as given by main()

		argv[]	char**	Array of strings containing the broken dowwm
				command line argv[0] contains the program name.

		allopts	char*	String containing all option characters
				without leading '-'. Options that may comprise
				a value must be followed by a colon.
				Example: "abc:de:f"
                                Options not listed in allopts are treated as
				value-less options.

		opt		Option to scan for (character) or a negative
				integer value starting with -1 to scan for 
				an argument.

		defval	char*	value to be returned if an option is specified
				without a value. Meaningless with parameters

		notfound char*	value to be returned if the option/parameter
				is not found during the scan. Returned also
				if a requested parameter is not present.
*/

char *getarg (int argc, char *argv[], char *allopts, int opt, 
              char *defval, char *notfound)
{
    int i;
    char *ptr, *otv;

    if (opt >= 0) {
        if ((otv=strchr(allopts,opt)) == NULL) 
            otv = "";
        else
            otv++;

        for (i = 1; i < argc; i++) {
            ptr = argv[i];
            if (*ptr++ == '-') {
                if (*ptr == '\0') continue;	/* dash-only ignored */
                if (*ptr++ == (char)opt) {
                    if (*otv != ':') return (defval);
                    if (*ptr == '\0') {
                        if (i == (argc-1)) {

                            return (defval);
                        }
                        else {
                            ptr = argv[i+1];
                            if (*ptr == '-') 
                                return (defval);
                            else
                                return (ptr);
                        }
                    }
                    else {
                        return (ptr);
                    }
                }
            }
        }
        return (notfound);
    }
    else {
        int last;

        for (i = 1, last = 0; i < argc; i++) {
            ptr = argv[i];
            if (*ptr++ != '-') break;	/* option list stops with first argument */
            if (*ptr == '\0') break;	/* dash-only treated as an argument */
            last = i;
            if ((otv=strchr(allopts,*ptr++)) != NULL)
                if (*(++otv) == ':') 
                    if (*ptr == '\0')
                        if (i < (argc-1))
                            if (*argv[i+1] != '-') {
                                last++;
                                i++;
                            }
        }
        if (last-opt >= argc) return (notfound);
        return (argv[last-opt]);
    }
}


int main (int argc, char ** argv)

{
    char input[1024];
    char output[8192];  // max. theoretische Laenge wenn input auschl. tab's
    char *pi, *po;
    int inlen, outlen, postscript = 0, help;
    FILE *fin, *fout;
    char *allopts = "p?", *infile, *outfile;

    postscript = atoi(getarg(argc, argv, allopts,'p', "1", "0"));
    help       = atoi(getarg(argc, argv, allopts,'?', "1", "0"));
    infile     =      getarg(argc, argv, allopts, -1, NULL, NULL);
    outfile    =      getarg(argc, argv, allopts, -2, NULL, NULL);

    if (help) {
        fprintf(stderr, "\nUsage is: asa [-p] [?] [infile] [outfile]\n"
                        "    -p        postscript mode - honours '+' control\n"
                        "    -?        print this help text and exit\n"
                        "    infile    input file. if empty or == '-', then\n"
                        "              pipe input via stdin is assumed\n"
                        "    outfile   output file. if empty or == '-', then\n"
                        "              pipe output via stdout is assumed\n\n");
        return 1;
    }
    
    fin = stdin;
    fout = stdout;

    if (infile && strcmp(infile, "-")) {
        if ((fin = fopen(infile, "r")) == NULL) {
            fprintf(stderr, "Cannot open input file '%s': %m\n", infile);
            return 2;
        }
    }

    if (outfile && strcmp(outfile, "-")) {
        if ((fout = fopen(outfile, "w")) == NULL) {
            fprintf(stderr, "Cannot open output file '%s': %m\n", outfile);
            return 3;
        }
    }

    while (fgets(input, sizeof(input), fin) != NULL) {
        inlen = strlen(input);
        input[inlen-1] = '\0';  // Newline am Stringende entfernen

// Tabulatorzeichen expandieren - wir nehmen eine Schrittweite von 8 an!

        for (pi=&input[1], po=output, outlen=0; *pi != '\0'; pi++) {
            if (*pi == '\t') {
                do {
                    *po++ = ' ';
                    outlen++;
                } while ((outlen%8) != 7);
            }
            else {
                *po++ = *pi;
                outlen++;
            }
        }
        *po = '\0';

        switch (input[0]) {
            default:            // Catch-all
            case ' ':           // Normale Zeile: Linefeed + Return
                fprintf(fout, "\n%s\r", output);
                break;
            case '0':           // Doppelter Zeilenvorschub
                fprintf(fout, "\n\n%s\r", output);
                break;
            case '1':           // Neue Seite (^L)
                fprintf(fout, "\f%s\r", output);
                break;
            case '+':           // Mehrfachdruck:
                                // muesste eigentlich die vorangegangene Zeile
                                // noch einmal ueberdrucken. Aber das geht nur
                                // mit Zeilen-/Nadeldruckern. Bei Bilschirm-
                                // darstellung und Laserdruckern klappt das
                                // nicht. Allerdings geht's mit PostScript
                                // Deshalb: mit Schalter -p wird die Zeile
                                // gedruckt, sonst einfach weggelassen!
                if (postscript)
                    fprintf(fout, "%s\r", output);
                break;
        }
    }
    fprintf(fout, "\n");  // Abschluss des Ausgabetextes
    fclose (fin);
    fclose (fout);
    return 0;
}
